/******************************************************************
*        PoLAR (Portable Library for Augmented Reality)           *
*     Copyright (C) 2015 Inria, Université de Lorraine, CNRS      *
*                                                                 *
* This library is free software; you can redistribute it and/or   *
* modify it under the terms of the GNU General Public License     *
* as published by the Free Software Foundation; either version 3  *
* of the License, or (at your option) any later version.          *
*                                                                 *
* This library is distributed in the hope that it will be useful, *
* but WITHOUT ANY WARRANTY; without even the implied warranty of  *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the    *
* GNU General Public License for more details.                    *
*                                                                 *
*                                                                 *
* You should have received a copy of the GNU General Public       *
* License along with this library.                                *
* If not, see <http://www.gnu.org/licenses/>.                     *
******************************************************************/

#ifndef _MYVIEWER_
#define _MYVIEWER_

#include <QFileDialog>
#include <QMessageBox>
#include <QObject>
#include <osg/MatrixTransform>
#include <osg/Geode>
#include <osg/ShapeDrawable>
#include <osgSim/SphereSegment>
#include <osg/io_utils>

#include <osgViewer/ViewerEventHandlers> // for the StatsHandler (frame rate...)
#include "Viewer.h"
#include "DrawableObject2D.h"
#include "BlendDialog.h"

//rotation callback
struct Callback : public osg::NodeCallback
{
    void update(PoLAR::Object3D *object)
    {
        object->rotate(0.005, 0., 1., 0.);
    }
};


class MyViewer : public PoLAR::Viewer
{
    Q_OBJECT
public:
    MyViewer(QWidget * parent = 0, const char * name = 0, WindowFlags f = 0):
        PoLAR::Viewer(parent, name, f, true), // MyViewer inherits from PoLAR::Viewer
        // Help messageBox creation
        mb(QMessageBox::Information,
           QString("Help: shortkeys"),
           QString("Shortkeys:\n"),
           QMessageBox::NoButton,
           this),
        _blendDialog(this)
    {
        // Add native OpenSceneGraph stats handler
        addEventHandler(new osgViewer::StatsHandler);
        mb.setWindowModality(Qt::NonModal);
    }

    int addObject3D(PoLAR::Object3D *o)
    {
        // Add an Object3D to this viewer
        int i = Viewer::addObject3D(o);

        // Set the blend dialog to be activ on the added object
        _blendDialog.setObject(o);
        return i;
    }

public slots:
    // add key bindings
    virtual void keyPressEvent(QKeyEvent *event)
    {
        
        switch(event->key())
        {
        case Qt::Key_Q:
        {
            close();
            break;
        }
        case Qt::Key_F1:
        {
            osg::ref_ptr<PoLAR::Object3D> o=getObject3D(0);
            osg::ref_ptr<PoLAR::Object3D> o2=getObject3D(1);
            if (o.valid())
            { // toggle the display of the Object3D
                if (o->isDisplayed())
                    {
                        o->setDisplayOff();
                        o2->setDisplayOff();
                    }
                else
                {
                  o->setDisplayOn();
                  o2->setDisplayOn();  
                } 
            }
            break;
        }
        case Qt::Key_F3:
        {
            osg::ref_ptr<PoLAR::Object3D> o=getObject3D(1); 
            //o->setUpdateCallback(new Callback);
            o->rotate(0.01, 0., 0., 1.);
            break;
        }
        default: Viewer::keyPressEvent(event);
        }
    }
private:
    QMessageBox mb;
    PoLAR::BlendDialog _blendDialog;
};

#endif // _MYVIEWER_
