#include <QtWidgets/QApplication> // for Qt functions
#include "MyViewer.h" // PoLAR::Viewer header
#include "Image.h"
#include "Util.h"

#include <iostream>
#include <QApplication>
#include <osg/Geode>
#include <osg/Geometry>
#include <osgDB/ReadFile>
#include <osgUtil/SmoothingVisitor>
#include <osgUtil/Optimizer>
#include <osg/io_utils>

int main(int argc, char** argv)
{
    // Create the Qt application
    QApplication app(argc, argv);
    //PoLAR::Viewer viewer(0, "Viewer", 0, true);
    MyViewer viewer (0, "Viewer", 0);
    osg::ref_ptr<PoLAR::Image<unsigned char> > myImage;

    myImage = new PoLAR::Image<unsigned char>("data/fond.png", true, 1);
    viewer.setBgImage(myImage);
    viewer.bgImageOn();

    int width = myImage->getWidth();
    int height = myImage->getHeight();
    viewer.resize(width, height);



    osgUtil::SmoothingVisitor smooth;
    osgUtil::Optimizer optimizer;

    //chargement de la chaise
    osg::ref_ptr<osg::Node> loadedModel2;
    loadedModel2 = osgDB::readNodeFile("data/plastic_chair.obj");
    optimizer.optimize(loadedModel2.get());
    loadedModel2->accept(smooth);
    osg::ref_ptr<PoLAR::Object3D> obj2 = new PoLAR::Object3D(loadedModel2.get(), true, true);

    obj2->scale(0.5f);
    obj2->translate(-42,85,-100);
    obj2->rotate(M_PI/2, 0, 0, 1);

    viewer.addObject3D(obj2.get());
    viewer.setTrackNode(obj2.get());

    //définition de la projection
    PoLAR::Viewer::ProjectionType pt=PoLAR::Viewer2D3D::VISION;
    osg::Matrixd P = PoLAR::Util::readProjectionMatrix("data/mat.proj");
    viewer.setProjection(P, pt);

    //phantom table :
    osg::ref_ptr<osg::Geode> tableGeode = new osg::Geode;
    osg::ref_ptr<osg::ShapeDrawable> shape;
    shape = new osg::ShapeDrawable(new osg::Box(osg::Vec3(1.0f, 2.5f, 0.0f), 7.0f, 12.0f, 0.00001f));
    tableGeode->addDrawable(shape.get());
    osg::ref_ptr<PoLAR::Object3D> object3D = new PoLAR::Object3D(tableGeode.get());
    object3D->setName("Table");
    object3D->setPhantomOn();
    viewer.addObject3D(object3D.get());
    object3D->setEditOn();

    viewer.addLightSource(5,-5, 5.5, true);
    

    //chargement de l'appareil photo (objet 3D) :
    osg::ref_ptr<osg::Node> loadedModel;
    loadedModel = osgDB::readNodeFile("data/reflex_camera.obj");
    

    optimizer.optimize(loadedModel.get());
    loadedModel->accept(smooth);
    osg::Matrixd m = osg::Matrixd::translate(1.2,-0.7,0);
    osg::ref_ptr<PoLAR::Object3D> obj = new PoLAR::Object3D(loadedModel.get(), true, true);
    obj->setTransformationMatrix(m);
    viewer.addObject3D(obj.get());
    viewer.setTrackNode(obj.get());
    obj->rotate(M_PI/7, 0, 0, 1);
    obj->scale(0.9f);


    // Show the widget
    viewer.show();
    app.connect( &app, SIGNAL(lastWindowClosed()), &app, SLOT(quit()) );
    return app.exec();
}